
#include <stdio.h>
#include <math.h>
#include "qcmatch.h"

#ifndef M_PI
#define M_PI	3.14159265358979323846
#endif

#define NODESIZE	 .01	/* Size of the circles used to represent nodes */
#define CENTX		0.5	/* The x value of the center of the circle */
#define CENTY		0.5	/* The y value of the center of the circle */
#define CIRRAD		0.5	/* The radius of the coordinate circle */

#define ROUNDOFF	 .000005
#define BIGSLOPE  1000000.0;

typedef struct {
	REAL	coord;
} NODE;

void
graphics_output( int n, NODE *nodes[], int colors[], int matching[] )
{
	int	kj;
	double	x1, x2, y1, y2; /* The coordinates of the nodes */
	double	x3, y3;		/* The coordinates of the intersection of the
				 * tangents. */
	double	radius;		/* The radius of an edge arc. */
	double	th1, th2;	/* The angles of the 2 nodes. */
	double	m1, m2;		/* The slopes of the tangents. */
	double	a1, a2;		/* Limits of the angle swept out by an arc. */


	printf("\nQCbegin\n");
	printf("PROOF\n");
				/* Draw the coordinate circle */
	printf("%6f  %6f  %6f  CIRCLE\n", CENTX, CENTY, CIRRAD);

				/* Draw the edge arcs */
	for (kj = 0; kj < n; kj++) {

				/* Unmatched or already-matched nodes */
		if (matching[kj] < 0 || kj >= matching[kj]) {
			continue;
		}

				/* Get the two angles in more convenient form. */
		th1 = nodes[kj]->coord;
		th2 = nodes[matching[kj]]->coord;

				/* Identical coordinates: draw no arc */
		if (th1 == th2) {
			continue;
		}

				/* Get coordinates of the two nodes. */
		x1 = CENTX + CIRRAD * cos(th1);
		y1 = CENTY + CIRRAD * sin(th1);
		x2 = CENTX + CIRRAD * cos(th2);
		y2 = CENTY + CIRRAD * sin(th2);

				/* If points are not 180 degrees apart */
		if (fabs((th2-th1) - M_PI) >= ROUNDOFF) {

					/* Get slopes of the tangent lines.*/
			m1 = tan(th1) ? -1/tan(th1) : BIGSLOPE;
			m2 = tan(th2) ? -1/tan(th2) : BIGSLOPE;

					/* Get coordinates of the intersection
					 * of the tangents.
					 */
			x3 = (y2 + m1*x1 - (y1 + m2*x2))/(m1-m2);
			y3 = m1 * (x3 - x1) + y1;

					/* Get the radius of the arc. */
			radius = pow((y3-y1)*(y3-y1) + (x3-x1)*(x3-x1), 0.5);

					/* Get the arc angles. */
			if (th2-th1 < M_PI) {	/* Sweep from th2 to th1. */
				a1 =  90 + (th2 * 180 / M_PI);
				a2 = 270 + (th1 * 180 / M_PI);

			} else {		/* Sweep from th1 to th2. */
				a1 =  90 + (th1 * 180 / M_PI);
				a2 = -90 + (th2 * 180 / M_PI);
			}

			printf("%6f  %6f  %6f   %6f   %6f   VARARC\n",
			       x3, y3, radius, a1, a2);

		} else {	/* 180 degs apart: join with line segment. */
			printf("%6f  %6f  %6f  %6f   LINE\n", x1, y1, x2, y2);
		}
	}

				/* Draw the node circles */
	for (kj = 0; kj < n; kj++) {

		printf("%6f  %6f  %6f  %sCIRCLE\n",
		       CENTX + CIRRAD * cos(nodes[kj]->coord),
		       CENTY + CIRRAD * sin(nodes[kj]->coord),
		       NODESIZE,
		       (colors[kj] == BLUE) ? "H" : "S");
	}

	printf("QCend\n");
}
