
#include <stdio.h>
#include <math.h>
#include "qcmatch.h"

#define squareit(A) ((A)*(A))

typedef struct {
	REAL	coord;
} NODE;

REAL	sqrtcost( void *nx, void *ny );
BOOLEAN Line_Omega( MNODE *x1, MNODE *x2, MNODE *x3, void *nodes[],
		COSTFUNC cost, int psi );

COSTFUNC	CostFunc  = sqrtcost;
OMEGAFUNC	OmegaFunc = Line_Omega;


REAL
sqrtcost( void *nx, void *ny )
{
	return sqrt(fabs(((NODE *)nx)->coord - ((NODE *)ny)->coord));
}

/* This version of omega works only for the linear/sqrt case */

BOOLEAN
Line_Omega( MNODE *x1, MNODE *x2, MNODE *x3, void *input[], COSTFUNC cost,
	int psi )
{
	REAL	del, eps;
	NODE	**nodes;

	nodes = (NODE **)input;

	del = psi * (x1->i - x2->i);
	eps = psi * (x2->i - x3->i);
	if ((eps <= 0) || (squareit(eps * ((nodes[x2->x]->coord -
					    nodes[x1->x]->coord) -
					   squareit(del))) <
			   squareit(del * ((nodes[x3->x]->coord -
					    nodes[x2->x]->coord) +
					   squareit(eps)))
			   && (del > 0))) {
		return TRUE;
	} else {
		return FALSE;
	}
}
